@@ -23,9 +23,12 @@ import enum
 import json
 import logging
 import pathlib
+import queue
 import re
+import shlex
 import typing
 
+import bluepy.btle
 import paho.mqtt.client
 import switchbot
 
@@ -43,17 +46,44 @@ _MQTTTopicLevel = typing.Union[str, _MQTTTopicPlaceholder]
 _MQTT_TOPIC_LEVELS_PREFIX = ["homeassistant"]  # type: typing.List[_MQTTTopicLevel]
 
 
+def _join_mqtt_topic_levels(
+    topic_levels: typing.List[_MQTTTopicLevel], mac_address: str
+) -> str:
+    return "/".join(
+        mac_address if l == _MQTTTopicPlaceholder.MAC_ADDRESS else typing.cast(str, l)
+        for l in topic_levels
+    )
+
+
 def _mac_address_valid(mac_address: str) -> bool:
     return _MAC_ADDRESS_REGEX.match(mac_address.lower()) is not None
 
 
+class _QueueLogHandler(logging.Handler):
+    """
+    logging.handlers.QueueHandler drops exc_info
+    """
+
+    # TypeError: 'type' object is not subscriptable
+    def __init__(self, log_queue: "queue.Queue[logging.LogRecord]") -> None:
+        self.log_queue = log_queue
+        super().__init__()
+
+    def emit(self, record: logging.LogRecord) -> None:
+        self.log_queue.put(record)
+
+
 class _MQTTCallbackUserdata:
     # pylint: disable=too-few-public-methods; @dataclasses.dataclass when python_requires>=3.7
     def __init__(
-        self, retry_count: int, device_passwords: typing.Dict[str, str]
+        self,
+        retry_count: int,
+        device_passwords: typing.Dict[str, str],
+        fetch_device_info: bool,
     ) -> None:
         self.retry_count = retry_count
         self.device_passwords = device_passwords
+        self.fetch_device_info = fetch_device_info
 
     def __eq__(self, other: object) -> bool:
         return isinstance(other, type(self)) and vars(self) == vars(other)
@@ -67,11 +97,15 @@ class _MQTTControlledActor(abc.ABC):
     def __init__(
         self, mac_address: str, retry_count: int, password: typing.Optional[str]
     ) -> None:
+        # alternative: pySwitchbot >=0.10.0 provides SwitchbotDevice.get_mac()
         self._mac_address = mac_address
 
     @abc.abstractmethod
     def execute_command(
-        self, mqtt_message_payload: bytes, mqtt_client: paho.mqtt.client.Client
+        self,
+        mqtt_message_payload: bytes,
+        mqtt_client: paho.mqtt.client.Client,
+        update_device_info: bool,
     ) -> None:
         raise NotImplementedError()
 
@@ -111,7 +145,10 @@ class _MQTTControlledActor(abc.ABC):
             password=userdata.device_passwords.get(mac_address, None),
         )
         actor.execute_command(
-            mqtt_message_payload=message.payload, mqtt_client=mqtt_client
+            mqtt_message_payload=message.payload,
+            mqtt_client=mqtt_client,
+            # consider calling update+report method directly when adding support for battery levels
+            update_device_info=userdata.fetch_device_info,
         )
 
     @classmethod
@@ -133,11 +170,8 @@ class _MQTTControlledActor(abc.ABC):
         payload: bytes,
         mqtt_client: paho.mqtt.client.Client,
     ) -> None:
-        topic = "/".join(
-            self._mac_address
-            if l == _MQTTTopicPlaceholder.MAC_ADDRESS
-            else typing.cast(str, l)
-            for l in topic_levels
+        topic = _join_mqtt_topic_levels(
+            topic_levels=topic_levels, mac_address=self._mac_address
         )
         # https://pypi.org/project/paho-mqtt/#publishing
         _LOGGER.debug("publishing topic=%s payload=%r", topic, payload)
@@ -178,7 +212,7 @@ class _ButtonAutomator(_MQTTControlledActor):
     ]
 
     def __init__(
-        self, mac_address: str, retry_count: int, password: typing.Optional[None]
+        self, mac_address: str, retry_count: int, password: typing.Optional[str]
     ) -> None:
         self._device = switchbot.Switchbot(
             mac=mac_address, password=password, retry_count=retry_count
@@ -188,7 +222,10 @@ class _ButtonAutomator(_MQTTControlledActor):
         )
 
     def execute_command(
-        self, mqtt_message_payload: bytes, mqtt_client: paho.mqtt.client.Client
+        self,
+        mqtt_message_payload: bytes,
+        mqtt_client: paho.mqtt.client.Client,
+        update_device_info: bool,
     ) -> None:
         # https://www.home-assistant.io/integrations/switch.mqtt/#payload_on
         if mqtt_message_payload.lower() == b"on":
@@ -228,18 +265,97 @@ class _CurtainMotor(_MQTTControlledActor):
         "state",
     ]
 
+    _MQTT_POSITION_TOPIC_LEVELS = _MQTT_TOPIC_LEVELS_PREFIX + [
+        "cover",
+        "switchbot-curtain",
+        _MQTTTopicPlaceholder.MAC_ADDRESS,
+        "position",
+    ]
+
+    @classmethod
+    def get_mqtt_position_topic(cls, mac_address: str) -> str:
+        return _join_mqtt_topic_levels(
+            topic_levels=cls._MQTT_POSITION_TOPIC_LEVELS, mac_address=mac_address
+        )
+
     def __init__(
-        self, mac_address: str, retry_count: int, password: typing.Optional[None]
+        self, mac_address: str, retry_count: int, password: typing.Optional[str]
     ) -> None:
+        # > The position of the curtain is saved in self._pos with 0 = open and 100 = closed.
+        # https://github.com/Danielhiversen/pySwitchbot/blob/0.10.0/switchbot/__init__.py#L150
         self._device = switchbot.SwitchbotCurtain(
-            mac=mac_address, password=password, retry_count=retry_count
+            mac=mac_address,
+            password=password,
+            retry_count=retry_count,
+            reverse_mode=True,
         )
         super().__init__(
             mac_address=mac_address, retry_count=retry_count, password=password
         )
 
+    def _report_position(self, mqtt_client: paho.mqtt.client.Client) -> None:
+        # > position_closed integer (Optional, default: 0)
+        # > position_open integer (Optional, default: 100)
+        # https://www.home-assistant.io/integrations/cover.mqtt/#position_closed
+        # SwitchbotCurtain.get_position() returns a cached value within [0, 100].
+        # SwitchbotCurtain.open() and .close() update the position optimistically,
+        # SwitchbotCurtain.update() fetches the real position via bluetooth.
+        # https://github.com/Danielhiversen/pySwitchbot/blob/0.10.0/switchbot/__init__.py#L202
+        self._mqtt_publish(
+            topic_levels=self._MQTT_POSITION_TOPIC_LEVELS,
+            payload=str(int(self._device.get_position())).encode(),
+            mqtt_client=mqtt_client,
+        )
+
+    def _update_position(self, mqtt_client: paho.mqtt.client.Client) -> None:
+        log_queue = queue.Queue(maxsize=0)  # type: queue.Queue[logging.LogRecord]
+        logging.getLogger("switchbot").addHandler(_QueueLogHandler(log_queue))
+        try:
+            self._device.update()
+            # pySwitchbot>=v0.10.1 catches bluepy.btle.BTLEManagementError :(
+            # https://github.com/Danielhiversen/pySwitchbot/blob/0.10.1/switchbot/__init__.py#L141
+            while not log_queue.empty():
+                log_record = log_queue.get()
+                if log_record.exc_info:
+                    exc = log_record.exc_info[1]  # type: typing.Optional[BaseException]
+                    if (
+                        isinstance(exc, bluepy.btle.BTLEManagementError)
+                        and exc.emsg == "Permission Denied"
+                    ):
+                        raise exc
+        except bluepy.btle.BTLEManagementError as exc:
+            if (
+                exc.emsg == "Permission Denied"
+                and exc.message == "Failed to execute management command 'le on'"
+            ):
+                raise PermissionError(
+                    "bluepy-helper failed to enable low energy mode"
+                    + " due to insufficient permissions."
+                    + "\nSee {}, {}, and {}.".format(
+                        "https://github.com/IanHarvey/bluepy/issues/313#issuecomment-428324639",
+                        "https://github.com/fphammerle/switchbot-mqtt/pull/31"
+                        + "#issuecomment-846383603",
+                        "https://github.com/IanHarvey/bluepy/blob/v/1.3.0/bluepy/bluepy-helper.c"
+                        + "#L1260",
+                    )
+                    + "\nInsecure workaround:"
+                    + "\n1. sudo apt-get install --no-install-recommends libcap2-bin"
+                    + "\n2. sudo setcap cap_net_admin+ep {}".format(
+                        shlex.quote(bluepy.btle.helperExe)
+                    )
+                    + "\n3. restart switchbot-mqtt"
+                    + "\nIn docker-based setups, you could use"
+                    + " `sudo docker run --cap-drop ALL --cap-add NET_ADMIN --user 0 …`"
+                    + " (seriously insecure)."
+                ) from exc
+            raise
+        self._report_position(mqtt_client=mqtt_client)
+
     def execute_command(
-        self, mqtt_message_payload: bytes, mqtt_client: paho.mqtt.client.Client
+        self,
+        mqtt_message_payload: bytes,
+        mqtt_client: paho.mqtt.client.Client,
+        update_device_info: bool,
     ) -> None:
         # https://www.home-assistant.io/integrations/cover.mqtt/#payload_open
         if mqtt_message_payload.lower() == b"open":
@@ -266,6 +382,8 @@ class _CurtainMotor(_MQTTControlledActor):
                 # https://www.home-assistant.io/integrations/cover.mqtt/#configuration-variables
                 # https://community.home-assistant.io/t/mqtt-how-to-remove-retained-messages/79029/2
                 self.report_state(mqtt_client=mqtt_client, state=b"")
+                if update_device_info:
+                    self._update_position(mqtt_client=mqtt_client)
         else:
             _LOGGER.warning(
                 "unexpected payload %r (expected 'OPEN', 'CLOSE', or 'STOP')",
@@ -295,11 +413,14 @@ def _run(
     mqtt_password: typing.Optional[str],
     retry_count: int,
     device_passwords: typing.Dict[str, str],
+    fetch_device_info: bool,
 ) -> None:
     # https://pypi.org/project/paho-mqtt/
     mqtt_client = paho.mqtt.client.Client(
         userdata=_MQTTCallbackUserdata(
-            retry_count=retry_count, device_passwords=device_passwords
+            retry_count=retry_count,
+            device_passwords=device_passwords,
+            fetch_device_info=fetch_device_info,
         )
     )
     mqtt_client.on_connect = _mqtt_on_connect
@@ -351,6 +472,13 @@ def _main() -> None:
         help="Maximum number of attempts to send a command to a SwitchBot device"
         " (default: %(default)d)",
     )
+    argparser.add_argument(
+        "--fetch-device-info",  # generic name to cover future addition of battery level etc.
+        action="store_true",
+        help="Report curtain motors' position on topic {} after sending stop command.".format(
+            _CurtainMotor.get_mqtt_position_topic(mac_address="MAC_ADDRESS")
+        ),
+    )
     args = argparser.parse_args()
     if args.mqtt_password_path:
         # .read_text() replaces \r\n with \n
@@ -372,4 +500,5 @@ def _main() -> None:
         mqtt_password=mqtt_password,
         retry_count=args.retry_count,
         device_passwords=device_passwords,
+        fetch_device_info=args.fetch_device_info,
     )
