@@ -28,6 +28,94 @@ import switchbot_mqtt
 # pylint: disable=too-many-arguments; these are tests, no API
 
 
+@pytest.mark.parametrize("mac_address", ["{MAC_ADDRESS}", "aa:bb:cc:dd:ee:ff"])
+def test_get_mqtt_position_topic(mac_address):
+    assert switchbot_mqtt._CurtainMotor.get_mqtt_position_topic(
+        mac_address=mac_address
+    ) == "homeassistant/cover/switchbot-curtain/{}/position".format(mac_address)
+
+
+@pytest.mark.parametrize(
+    "mac_address",
+    ("aa:bb:cc:dd:ee:ff", "aa:bb:cc:dd:ee:gg"),
+)
+@pytest.mark.parametrize(
+    ("position", "expected_payload"), [(0, b"0"), (100, b"100"), (42, b"42")]
+)
+def test__report_position(
+    caplog, mac_address: str, position: int, expected_payload: bytes
+):
+    with unittest.mock.patch(
+        "switchbot.SwitchbotCurtain.__init__", return_value=None
+    ) as device_init_mock, caplog.at_level(logging.DEBUG):
+        actor = switchbot_mqtt._CurtainMotor(
+            mac_address=mac_address, retry_count=7, password=None
+        )
+    device_init_mock.assert_called_once_with(
+        mac=mac_address,
+        retry_count=7,
+        password=None,
+        # > The position of the curtain is saved in self._pos with 0 = open and 100 = closed.
+        # > [...] The parameter 'reverse_mode' reverse these values, [...]
+        # > The parameter is default set to True so that the definition of position
+        # > is the same as in Home Assistant.
+        # https://github.com/Danielhiversen/pySwitchbot/blob/0.10.0/switchbot/__init__.py#L150
+        reverse_mode=True,
+    )
+    with unittest.mock.patch.object(
+        actor, "_mqtt_publish"
+    ) as publish_mock, unittest.mock.patch(
+        "switchbot.SwitchbotCurtain.get_position", return_value=position
+    ):
+        actor._report_position(mqtt_client="dummy")
+    publish_mock.assert_called_once_with(
+        topic_levels=[
+            "homeassistant",
+            "cover",
+            "switchbot-curtain",
+            switchbot_mqtt._MQTTTopicPlaceholder.MAC_ADDRESS,
+            "position",
+        ],
+        payload=expected_payload,
+        mqtt_client="dummy",
+    )
+    assert not caplog.record_tuples
+
+
+@pytest.mark.parametrize("position", ("", 'lambda: print("")'))
+def test__report_position_invalid(caplog, position):
+    with unittest.mock.patch(
+        "switchbot.SwitchbotCurtain.__init__", return_value=None
+    ), caplog.at_level(logging.DEBUG):
+        actor = switchbot_mqtt._CurtainMotor(
+            mac_address="aa:bb:cc:dd:ee:ff", retry_count=3, password=None
+        )
+    with unittest.mock.patch.object(
+        actor, "_mqtt_publish"
+    ) as publish_mock, unittest.mock.patch(
+        "switchbot.SwitchbotCurtain.get_position", return_value=position
+    ), pytest.raises(
+        ValueError
+    ):
+        actor._report_position(mqtt_client="dummy")
+    publish_mock.assert_not_called()
+
+
+def test__update_position():
+    with unittest.mock.patch("switchbot.SwitchbotCurtain.__init__", return_value=None):
+        actor = switchbot_mqtt._CurtainMotor(
+            mac_address="dummy", retry_count=21, password=None
+        )
+    with unittest.mock.patch(
+        "switchbot.SwitchbotCurtain.update"
+    ) as update_mock, unittest.mock.patch.object(
+        actor, "_report_position"
+    ) as report_position_mock:
+        actor._update_position(mqtt_client="client")
+    update_mock.assert_called_once_with()
+    report_position_mock.assert_called_once_with(mqtt_client="client")
+
+
 @pytest.mark.parametrize("mac_address", ["aa:bb:cc:dd:ee:ff", "aa:bb:cc:11:22:33"])
 @pytest.mark.parametrize("password", ["pa$$word", None])
 @pytest.mark.parametrize("retry_count", (2, 3))
@@ -45,6 +133,7 @@ import switchbot_mqtt
         (b"Stop", "switchbot.SwitchbotCurtain.stop"),
     ],
 )
+@pytest.mark.parametrize("report_position_upon_stop", [True, False])
 @pytest.mark.parametrize("command_successful", [True, False])
 def test_execute_command(
     caplog,
@@ -53,6 +142,7 @@ def test_execute_command(
     retry_count,
     message_payload,
     action_name,
+    report_position_upon_stop,
     command_successful,
 ):
     with unittest.mock.patch(
@@ -65,14 +155,16 @@ def test_execute_command(
             actor, "report_state"
         ) as report_mock, unittest.mock.patch(
             action_name, return_value=command_successful
-        ) as action_mock:
+        ) as action_mock, unittest.mock.patch.object(
+            actor, "_update_position"
+        ) as update_position_mock:
             actor.execute_command(
-                mqtt_client="dummy", mqtt_message_payload=message_payload
+                mqtt_client="dummy",
+                mqtt_message_payload=message_payload,
+                update_device_info=report_position_upon_stop,
             )
     device_init_mock.assert_called_once_with(
-        mac=mac_address,
-        password=password,
-        retry_count=retry_count,
+        mac=mac_address, password=password, retry_count=retry_count, reverse_mode=True
     )
     action_mock.assert_called_once_with()
     if command_successful:
@@ -106,6 +198,14 @@ def test_execute_command(
             )
         ]
         report_mock.assert_not_called()
+    if (
+        report_position_upon_stop
+        and action_name == "switchbot.SwitchbotCurtain.stop"
+        and command_successful
+    ):
+        update_position_mock.assert_called_once_with(mqtt_client="dummy")
+    else:
+        update_position_mock.assert_not_called()
 
 
 @pytest.mark.parametrize("mac_address", ["aa:bb:cc:dd:ee:ff"])
@@ -122,10 +222,12 @@ def test_execute_command_invalid_payload(
         )
         with unittest.mock.patch.object(actor, "report_state") as report_mock:
             actor.execute_command(
-                mqtt_client="dummy", mqtt_message_payload=message_payload
+                mqtt_client="dummy",
+                mqtt_message_payload=message_payload,
+                update_device_info=True,
             )
     device_mock.assert_called_once_with(
-        mac=mac_address, password=password, retry_count=7
+        mac=mac_address, password=password, retry_count=7, reverse_mode=True
     )
     assert not device_mock().mock_calls  # no methods called
     report_mock.assert_not_called()
@@ -157,7 +259,11 @@ def test_execute_command_bluetooth_error(caplog, mac_address, message_payload):
     ), caplog.at_level(logging.ERROR):
         switchbot_mqtt._CurtainMotor(
             mac_address=mac_address, retry_count=10, password="secret"
-        ).execute_command(mqtt_client="dummy", mqtt_message_payload=message_payload)
+        ).execute_command(
+            mqtt_client="dummy",
+            mqtt_message_payload=message_payload,
+            update_device_info=True,
+        )
     assert caplog.record_tuples == [
         (
             "switchbot",
